from abc import ABC
from gepref.preprocessor import GenericPreprocessor
from gepref.step import AbstractStep

class TextPreprocessor(GenericPreprocessor[str]):
    """
    Processes input text elements through several preprocessing steps

    :param steps: list of preprocessing steps
    :type steps: List[AbstractStep[str]]
    """

class AbstractTextStep(AbstractStep[str], ABC):
    """
    Interface that defines the behavior of a text preprocessing step.
    It requires to implement the `call` method.
    """
    ...



import pytest
from gepref_text.normalization import (
        LowerStep, UpperStep, TitleStep, UnidecodeStep
        )

@pytest.mark.parametrize("text", [
    "Sample Text",
    "UPPERCASE TEXT",
    "lowercase text"
    ])
def test_lower_step(text: str):
    norm_text = LowerStep()(text)
    assert norm_text == text.lower()

@pytest.mark.parametrize("text", [
    "Sample Text",
    "UPPERCASE TEXT",
    "lowercase text"
    ])
def test_upper_step(text: str):
    norm_text = UpperStep()(text)
    assert norm_text == text.upper()

@pytest.mark.parametrize("text", [
    "Sample Text",
    "UPPERCASE TEXT",
    "lowercase text"
    ])
def test_title_step(text: str):
    norm_text = TitleStep()(text)
    assert norm_text == text.title()

@pytest.mark.parametrize("text, norm", [
    ("Gabriel García Márquez", "Gabriel Garcia Marquez"),
    ("También está considerado", "Tambien esta considerado")
    ])
def test_title_step(text: str, norm: str):
    norm_text = UnidecodeStep()(text)
    assert norm_text == norm



from unidecode import unidecode
from gepref_text.base import AbstractTextStep

class LowerStep(AbstractTextStep):
    """
    Converts the text to lowercase.
    """

    def call(self, data: str) -> str:
        """
        Preprocess an input text element.

        :param data: Input text element.
        :type data: str
        :returns: Preprocessed data.
        :rtype: str
        """
        return data.lower()

class UpperStep(AbstractTextStep):
    """
    Converts the text to uppercase.
    """

    def call(self, data: str) -> str:
        """
        Preprocess an input text element.

        :param data: Input text element.
        :type data: str
        :returns: Preprocessed data.
        :rtype: str
        """
        return data.upper()

class TitleStep(AbstractTextStep):
    """
    Converts the text to title.
    """

    def call(self, data: str) -> str:
        """
        Preprocess an input text element.

        :param data: Input text element.
        :type data: str
        :returns: Preprocessed data.
        :rtype: str
        """
        return data.title()

class UnidecodeStep(AbstractTextStep):
    """
    Performs unidecode text normalization.
    """

    def call(self, data: str) -> str:
        """
        Preprocess an input text element.

        :param data: Input text element.
        :type data: str
        :returns: Preprocessed data.
        :rtype: str
        """
        return unidecode(data)
